% Required for the Matlab code example for Algorithm 1 presented in "Sparse 
% Fast DCT for Vectors with One-block Support" by Sina Bittens and Gerlind
% Plonka.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ supp_struct, mu1, m1, mu2, m2 ] = find_supp_2(y,j,J,mu,m,epsilon)
%FIND_SUPP_2 Finds the support structure of y=y^(j+1) if y^(j) has a 
%one-block support with length m<=2^(j-1).
% supp_struct==1 if y has a one-block support, supp_struct==2 if y has a
% two-block support and supp_struct==3 if it has a two-block support with 
% possibly different block lengths (only possible for j=J-1).

% Since we do not know whether supp_struct==3, we set the parameters of the
% second, possibly short or longer, block to 0.
mu2=0;
m2=0;
% supp_struct==3 is only possible for j==J-1.
if j<J-1
    % If 2^(j-1)-1,2^(j-1) are contained in S^(j), then y^(j+1) has a
    % two-block support.
    if (mu+m-1<=2^j-1 && ((2^(j-1)-1>=mu) && (2^(j-1)-1<=mu+m-1) && (2^(j-1)>=mu) && (2^(j-1)<=mu+m-1) ))  ||  (mu+m-1>2^j-1 && ((2^(j-1)-1>=mu  || 2^(j-1)-1<=mod(mu+m-1,2^j)) && (2^(j-1)>=mu  || 2^(j-1)<=mod(mu+m-1,2^j)) ))
        supp_struct=2;
        y_0=y((mu:2^j-1-mu)+1);
        % Find the first support index of the first block.
        T=find(y_0>epsilon)-1+mu;
        mu1=T(1);
        m1=T(end)-T(1)+1;
    % If 0, 2^j-1 are contained in S^(j), then y^(j+1) has a one-block
    % support.
    elseif (mu+m-1<=2^j-1 && ((0>=mu) && (0<=mu+m-1) && (2^j-1>=mu) && (2^j-1<=mu+m-1) ))  ||  (mu+m-1>2^j-1 && ((0>=mu  || 0<=mod(mu+m-1,2^j)) && (2^j-1>=mu  || 2^j-1<=mod(mu+m-1,2^j)) ))
        supp_struct=1;
        % Find the first support index of the block.
        e_0=sum(y((mu:mu+m-1)+1));
        e_1=sum(y(mod((mu:mu+m-1)+2^j+1,2^(j+1))+1));
        if e_0>e_1
            mu1=mu;
        elseif e_0<e_1
            mu1=mu+2^j;
        end
        m1=m;
    end
else 
    % If j=J-1, we have the same cases as before, but it might also happen
    % that y has a two-block support with two blocks of different lengths.
    % If 2^(j-1)-1,2^(j-1) are contained in S^(j), then y^(j+1) has a 
    % two-block support.
    if (mu+m-1<=2^j-1 && ((2^(j-1)-1>=mu) && (2^(j-1)-1<=mu+m-1) && (2^(j-1)>=mu) && (2^(j-1)<=mu+m-1) ))  ||  (mu+m-1>2^j-1 && ((2^(j-1)-1>=mu  || 2^(j-1)-1<=mod(mu+m-1,2^j)) && (2^(j-1)>=mu  || 2^(j-1)<=mod(mu+m-1,2^j)) ))
        supp_struct=2;
        y_0=y((mu:2^j-1-mu)+1);
        % Find the first support index of the first block.
        T=find(y_0>epsilon)-1+mu;
        mu1=T(1);
        m1=T(end)-T(1)+1;
    % If 0, 2^j-1 are contained in S^(j), then y^(j+1) has a two-block
    % support with possibly different block lengths, and one of the blocks
    % mught be empty.
    elseif (mu+m-1<=2^j-1 && ((0>=mu) && (0<=mu+m-1) && (2^j-1>=mu) && (2^j-1<=mu+m-1) ))  ||  (mu+m-1>2^j-1 && ((0>=mu  || 0<=mod(mu+m-1,2^j)) && (2^j-1>=mu  || 2^j-1<=mod(mu+m-1,2^j)) ))
        y_0=y((mu:2^j-1)+1);
        y_1=y((2^j+mu:2^(j+1)-1)+1);
        % Find the relevant entries of y^(j+1).
        T_0=find(y_0>epsilon)-1+mu;
        T_1=find(y_1>epsilon)-1+2^j+mu;
        % Not both index sets T_0 and T_1 can be empty.
        if isempty(T_0)==1 
            % If T_0 is empty, then y^(j+1) has only a one-block support 
            % and 0,2^J-1 are contained in it.
            supp_struct=1;
            mu1=T_1(1);
            m1=2*(2^(j+1)-mu1);
        elseif isempty(T_1)==1
            % If T_1 is empty, then y^(j+1) has only a one-block support
            % and 2^(J-1)-1, 2^(J-1) are contained in it.
            supp_struct=1;
            mu1=T_0(1);
            m1=2*(2^j-mu1);            
        else
            % If neither set is empty, then y^(j+1) has a two-block support
            % with two blocks that might have different lengths. This can 
            % only happen if 0, 2^(J-1)-1 are contained in the support of 
            % x.
            supp_struct=3;            
            mu1=T_0(1);
            m1=2*(2^j-mu1);
            mu2=T_1(1);
            m2=2*(2^J-mu2);
        end
    end
end
        
end

